*!*	Definition of the subclass of GpImage
*!*	Author : Cesar Chalom - cchalom@hotmail.com
*!*	Description : adds functions to Change or Remove Metadata Information of Images
*!*	              fixes GetPropertyItem method
*!*	Special thanks to Walter Nicholls, that did an awesome job creating _gdiplus.vcx

*!*	LICENSE :
*!*	This class is free; use it on your own risk. 
*!*	If you use this source code in a product, acknowledgment is not required but would be appreciated. 
*!*	Nobody is authorized to sell the class unless specific permission is given by the author.

*!*	More Information about Image Property Tag Type Constants on MSDN
*!*	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdicpp/GDIPlus/GDIPlusreference/constants/imagepropertytagtypeconstants.asp
*!*	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdicpp/GDIPlus/GDIPlusreference/constants/imagepropertytagconstants.asp

*!*	PropertyItem Descriptions
*!*	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdicpp/GDIPlus/GDIPlusreference/constants/imagepropertytagconstants/propertyitemdescriptions.asp

*!*	More great Tags Information at :
*!*	http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/EXIF.html
*!* http://pel.sourceforge.net/doc/PEL/PelTag.html


DEFINE CLASS MyGpImage as GpImage OF (HOME() + "ffc\_gdiplus.vcx")

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
PROCEDURE RemovePropertyItem
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
LPARAMETERS tnPropID as Integer
* tnPropID = Property ID to remove from image (integer)

if This.gdipHandle==0
	This.gdipStatus = 1 && GDIPLUS_STATUS_GenericError
	return cast(null as I)
endif

IF !(VARTYPE(m.tnPropID)='N')
	This.gdipStatus = 2 && GDIPLUS_STATUS_InvalidParameter
	return cast(null as I)
ENDIF

DECLARE LONG GdipRemovePropertyItem IN GDIPLUS LONG nImage, LONG propId

This.gdipStatus = GdipRemovePropertyItem(This.gdipHandle, m.tnPropID)
IF 0 != This.gdipStatus
	RETURN null
ENDIF 

ENDPROC



*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
PROCEDURE SetPropertyItem
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
LPARAMETERS tnPropID as Integer, tuNewValue, tnPropType
* tnPropID   = Property ID to add or change to Image (integer)
* tuNewValue = New value to store.
*     Depending on the type of metadata to store tuNewValue must be :
*     ASCII - String
*     Short, Long or SLong - Integer
*     Rational or SRational - String containing a fraction, eg. "1/12"
* tnPropType = optional, PropertyType Constant (Integer)
*     if not passed, this procedure will try to guess it

#DEFINE PropertyTagTypeByte      	 1 
#DEFINE PropertyTagTypeASCII     	 2 
#DEFINE PropertyTagTypeShort     	 3 
#DEFINE PropertyTagTypeLong      	 4 
#DEFINE PropertyTagTypeRational      5 
#DEFINE PropertyTagTypeUndefined     7 
#DEFINE PropertyTagTypeSLONG         9 
#DEFINE PropertyTagTypeSRational    10 

if This.gdipHandle==0
	This.gdipStatus = 1 && GDIPLUS_STATUS_GenericError
	return cast(null as I)
endif

IF NOT (VARTYPE(m.tnPropID)='N') OR NOT VARTYPE(tuNewValue) $ "CN"
	This.gdipStatus = 2 && Gdiplus_STATUS_InvalidParameter
	return cast(null as I)
ENDIF

DECLARE INTEGER GlobalAlloc in kernel32.dll integer nFlags, integer nSize
DECLARE INTEGER GlobalFree  in kernel32.dll integer nHandle
DECLARE INTEGER GlobalSize IN kernel32 INTEGER hMem

DECLARE INTEGER GdipGetPropertyItemSize in gdiplus.dll ;
	INTEGER nImage, integer nPropID, integer @ nBufSize
DECLARE INTEGER GdipGetPropertyItem in gdiplus.dll ;
	INTEGER nImage, integer nPropID, integer nBufSize, integer nBufferPtr
DECLARE INTEGER GdipSetPropertyItem IN gdiplus.dll; 
	INTEGER img, STRING @itm 

LOCAL lnPropertyTagType, lcNewValue


*!*	Check if the PropertyType Parameter was passed ;
*!*	If not, check if the property already exists in the image ;
*!*	If yes, get the propertyType from there, otherwise guess and make some tries
IF VARTYPE(tnPropType) = "N"
	lnPropertyTagType = tnPropType
ELSE 
	LOCAL lnBufferSize as integer, lnBufferPtr as Integer, lnStringPtr as integer
	LOCAL llGuess as Boolean 

	lnBufferSize = 0
	llGuess = .F.

	This.gdipStatus = GdipGetPropertyItemSize( This.gdipHandle, m.tnPropID, @lnBufferSize )

	DO CASE
	CASE This.gdipStatus <> 0 AND This.gdipStatus <> 19
		RETURN null

	CASE This.gdipStatus = 0 && Property found
		lnBufferPtr = GlobalAlloc( 0x0040, m.lnBufferSize ) && 0x40=GMEM_FIXED|GMEM_ZEROINIT
		IF 0==m.lnBufferPtr
			* Memory allocation error
			RETURN null
		ENDIF 

		This.gdipStatus = GdipGetPropertyItem( This.gdipHandle, m.tnPropID, m.lnBufferSize, m.lnBufferPtr )
		IF 0 != This.gdipStatus
			GlobalFree( m.lnBufferPtr )
			return null
		ENDIF

		* Now decode the parameter
		* PropertyItem
		* Offset	Size	Description
		*   0		 4 		ID (32 bit int)
		*	4		 4		Length of value 
		*	8		 4		Data type - GP_PROPERTYTAGTYPE_xxx
		*  12		 4		Pointer to value
		*  16 (probably)	The value itself

		lnPropertyTagType = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnBufferPtr+8,4))),0x80000000)
		GlobalFree( m.lnBufferPtr )

	CASE This.gdipStatus = 19 && Property NOT found
		llGuess = .T.
		DO CASE 
			CASE VARTYPE(tuNewValue) = "C" AND "/" $ tuNewValue
				lnPropertyTagType = 5 && PropertyTagTypeRational
			CASE VARTYPE(tuNewValue) = "C" AND NOT "/" $ tuNewValue
				lnPropertyTagType = 2 && PropertyTagTypeASCII
			CASE VARTYPE(tuNewValue) = "N"
				lnPropertyTagType = 4 && PropertyTagTypeLong
		ENDCASE 
	ENDCASE 
ENDIF

DO CASE 

CASE INLIST(m.lnPropertyTagType,0,6,8)	&& nothing
	RETURN null	&& warning: this is also error value

CASE m.lnPropertyTagType = PropertyTagTypeUndefined
	lcNewValue = tuNewValue

CASE m.lnPropertyTagType = PropertyTagTypeShort
	lcNewValue = BINTOC(tuNewValue, "2RS")

CASE m.lnPropertyTagType = PropertyTagTypeLong OR ;
		m.lnPropertyTagType = PropertyTagTypeSLONG         
	lcNewValue = BINTOC(tuNewValue, "4RS")

CASE m.lnPropertyTagType = PropertyTagTypeASCII
	lcNewValue = tuNewValue

CASE m.lnPropertyTagType = PropertyTagTypeByte
	lcNewValue = STRCONV(tuNewValue,5)

CASE m.lnPropertyTagType = PropertyTagTypeRational OR ;
		m.lnPropertyTagType = PropertyTagTypeSRational
	
	IF NOT (VARTYPE(tuNewValue) = "C" AND "/" $ tuNewValue)
		This.gdipStatus = 2 && GDIPLUS_STATUS_InvalidParameter
		return cast(null as I)
	ENDIF 
	
	LOCAL lnPosBar
	lnPosBar = AT("/",tuNewValue)
	lcNewValue = BINTOC(VAL(LEFT(tuNewValue,lnPosBar-1)), "RS") + ;
				BINTOC(VAL(RIGHT(tuNewValue,LEN(tuNewValue) - lnPosBar)), "RS")

ENDCASE

	*!* Create and allocate array
	LOCAL lnBufferPtr, lnLength, lcItem
	lcString = lcNewValue
	lnBufferPtr = 0

	#DEFINE GMEM_FIXED 0
    LOCAL lnSize
    lcString = lcString + Chr(0)
    lnSize = Len(lcString)
	lnBufferPtr = GlobalAlloc (GMEM_FIXED, lnSize)
	IF m.lnBufferPtr = 0
		* Memory allocation error
		error _GDIPLUS_LOC_MALLOCFAIL
		return null
	ENDIF
	SYS(2600,m.lnBufferPtr ,lnSize, lcString)
	lnLength = IIF(m.lnBufferPtr=0, 0, GlobalSize(m.lnBufferPtr))

	lcItem = BINTOC(m.tnPropId, "RS") + ;
				BINTOC(m.lnLength, "RS") + ;
				BINTOC(m.lnPropertyTagType, "RS") + ;
				BINTOC(m.lnBufferPtr, "RS")

This.gdipStatus = GdipSetPropertyItem(This.gdipHandle, m.lcItem)
IF 0 != This.gdipStatus
	RETURN null
ENDIF 

IF m.lnBufferPtr <> 0
   = GlobalFree (m.lnBufferPtr)
ENDIF

ENDPROC 


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
PROCEDURE GetPropertyItem
*!* Comments by Cesar Chalom
*!*	This procedure is an exact copy of the original code from Walter Nichols in _gdiplus.vcx
*!*	The only modification are in retrieving values when PropertyTagType	= Byte
*!*                           also, enables retrieval of PropertyTagUndefined
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
lparameters tnPropID as Integer

if This.gdipHandle==0
	error _GDIPLUS_NOGDIPOBJECT_LOC
	This.gdipStatus = 1 && GDIPLUS_STATUS_GenericError
	return cast(null as I)
endif

if !(vartype(m.tnPropID)='N')
	This.gdipStatus = 2 && GDIPLUS_STATUS_InvalidParameter
	error 11 && Function argument
	return cast(null as I)
endif

declare integer GlobalAlloc in kernel32.dll integer nFlags, integer nSize
declare integer GlobalFree in kernel32.dll integer nHandle
declare integer lstrlenA in kernel32.dll as __win32_lstrlenA_ptr integer

declare integer GdipGetPropertyItemSize in gdiplus.dll ;
	integer nImage, integer nPropID, integer @ nBufSize
declare integer GdipGetPropertyItem in gdiplus.dll ;
	integer nImage, integer nPropID, integer nBufSize, integer nBufferPtr
local lnBufferSize as integer, lnBufferPtr as Integer, lnStringPtr as integer
lnBufferSize = 0

This.gdipStatus = GdipGetPropertyItemSize( This.gdipHandle, m.tnPropID, @lnBufferSize )
if 0 != This.gdipStatus
	return null
endif

lnBufferPtr = GlobalAlloc( 0x0040, m.lnBufferSize ) && 0x40=GMEM_FIXED|GMEM_ZEROINIT
if 0==m.lnBufferPtr
	* Memory allocation error
	error 'Memory allocation failed' && _GDIPLUS_MALLOCFAIL_LOC
	return null
endif

This.gdipStatus = GdipGetPropertyItem( This.gdipHandle, m.tnPropID, m.lnBufferSize, m.lnBufferPtr )
if 0 != This.gdipStatus
	GlobalFree( m.lnBufferPtr )
	return null
endif

* Now decode the parameter
* PropertyItem
* Offset	Size	Description
*   0		 4 		ID (32 bit int)
*	4		 4		Length of value 
*	8		 4		Data type - GP_PROPERTYTAGTYPE_xxx
*  12		 4		Pointer to value
*  16 (probably)	The value itself
local lnPropertyTagType, lnValueLen, lnValuePtr, lvReturn
lnPropertyTagType = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnBufferPtr+8,4))),0x80000000)

lnValueLen = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnBufferPtr+4,4))),0x80000000)
lnValuePtr = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnBufferPtr+12,4))),0x80000000)

do case

case inlist(m.lnPropertyTagType,0,6,8)	&& nothing
	lvReturn = null	&& warning: this is also error value

case m.lnPropertyTagType = 7   && undefined
	&& Each component will be a byte with no associated interpretation.
	lvReturn = TRANSFORM(STRTRAN(STRCONV( sys(2600,m.lnValuePtr,m.lnValueLen) ,6),CHR(0),""))

case 1 == m.lnPropertyTagType	&& Byte
	lvReturn = TRANSFORM(STRTRAN(STRCONV( sys(2600,m.lnValuePtr,m.lnValueLen) ,6),CHR(0),""))
***	lvReturn = asc( sys(2600,m.lnValuePtr,m.lnValueLen))

case 2 == m.lnPropertyTagType	&& ASCII string (_not_ unicode)
	* trim null byte off end, but only if there! (It *should* be, but..)
	lvReturn = strextract(sys(2600,m.lnValuePtr,m.lnValueLen),'',chr(0),1,2)
case 3 == m.lnPropertyTagType	&& Short (16 bit)
	lvReturn = asc(sys(2600,m.lnValuePtr,1))+256*asc(sys(2600,m.lnValuePtr+1,1))
case 4 == m.lnPropertyTagType	&& Long (32 bit)
	lvReturn = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr,4))),0x80000000)
case 5 == m.lnPropertyTagType	&& Rational(two 32 bit)
	* return as string
	lvReturn = ltrim(str( bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr,4))),0x80000000) )) ;
		+ '/' ;
		+ ltrim(str( bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr+4,4))),0x80000000) ))
case 9 == m.lnPropertyTagType	&& signed Long (32 bit)
	lvReturn = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr,4))),0x80000000)
	if m.lvReturn>0x7FFFFFFF
		lvReturn = m.lvReturn - 0x100000000
	endif
case 10 == m.lnPropertyTagType	&& signed rational a/b
	local lnNum, lnDen
	lnNum = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr,4))),0x80000000)
	lnDen = bitxor(ctobin(chrtran('ABCD','DCBA',sys(2600,m.lnValuePtr+4,4))),0x80000000)
	if m.lnNum >0x7FFFFFFF
		lnNum = m.lnNum - 0x100000000
	endif
	if m.lnDen >0x7FFFFFFF
		lnDen = m.lnDen - 0x100000000
	endif
	lvReturn = ltrim(str(m.lnNum)) +'/'+ltrim(str(m.lnDen))

otherwise
	GlobalFree( m.lnBufferPtr )
	error 'Unknown or invalid property tag type' && _GDIPLUS_BADPROPERTYTAGTYPE_LOC
	return null
endcase
	
GlobalFree( m.lnBufferPtr )

return m.lvReturn
ENDPROC 

ENDDEFINE